@@ -30,6 +30,7 @@ import java.io.IOException;
 import java.util.concurrent.TimeUnit;
 
 import org.apache.bookkeeper.bookie.Bookie;
+import org.apache.bookkeeper.bookie.BookieException;
 import org.apache.bookkeeper.proto.BookkeeperProtocol.ReadLacRequest;
 import org.apache.bookkeeper.proto.BookkeeperProtocol.ReadLacResponse;
 import org.apache.bookkeeper.proto.BookkeeperProtocol.Request;
@@ -75,9 +76,12 @@ class ReadLacProcessorV3 extends PacketProcessorBaseV3 implements Runnable {
         } catch (Bookie.NoLedgerException e) {
             status = StatusCode.ENOLEDGER;
             logger.debug("No ledger found while performing readLac from ledger: {}", ledgerId, e);
-        } catch (IOException e) {
+        } catch (BookieException.DataUnknownException e) {
+            status = StatusCode.EUNKNOWNLEDGERSTATE;
+            logger.error("Ledger {} in unknown state and cannot serve reacLac requests", ledgerId, e);
+        } catch (BookieException | IOException e) {
             status = StatusCode.EIO;
-            logger.error("IOException while performing readLac from ledger: {}", ledgerId);
+            logger.error("IOException while performing readLac from ledger: {}", ledgerId, e);
         } finally {
             ReferenceCountUtil.release(lac);
         }
@@ -90,10 +94,10 @@ class ReadLacProcessorV3 extends PacketProcessorBaseV3 implements Runnable {
         } catch (Bookie.NoLedgerException e) {
             status = StatusCode.ENOLEDGER;
             logger.debug("No ledger found while trying to read last entry: {}", ledgerId, e);
-        } catch (Bookie.NoEntryException e) {
-            status = StatusCode.ENOENTRY;
-            logger.warn("No Entry found while trying to read last entry: {}", ledgerId, e);
-        } catch (IOException e) {
+        } catch (BookieException.DataUnknownException e) {
+            status = StatusCode.EUNKNOWNLEDGERSTATE;
+            logger.error("Ledger in an unknown state while trying to read last entry: {}", ledgerId, e);
+        } catch (BookieException | IOException e) {
             status = StatusCode.EIO;
             logger.error("IOException while trying to read last entry: {}", ledgerId, e);
         } finally {
