@@ -30,7 +30,7 @@ import io.netty.buffer.ByteBufAllocator;
 import io.netty.buffer.ByteBufUtil;
 import io.netty.buffer.Unpooled;
 import io.netty.buffer.UnpooledByteBufAllocator;
-
+import java.io.IOException;
 import java.nio.charset.Charset;
 import java.util.concurrent.BrokenBarrierException;
 import java.util.concurrent.CountDownLatch;
@@ -144,7 +144,7 @@ public class WriteCacheTest {
     }
 
     @Test
-    public void testEmptyCache() {
+    public void testEmptyCache() throws IOException {
         WriteCache cache = new WriteCache(allocator, 1024 * 1024, 16 * 1024);
 
         assertEquals(0, cache.count());
@@ -222,7 +222,7 @@ public class WriteCacheTest {
     }
 
     @Test
-    public void testLedgerDeletion() {
+    public void testLedgerDeletion() throws IOException {
         WriteCache cache = new WriteCache(allocator, 1024 * 1024, 16 * 1024);
 
         ByteBuf entry = Unpooled.buffer(1024);
@@ -306,4 +306,21 @@ public class WriteCacheTest {
         }
         assertFalse(cache.hasEntry(ledgerId, 48));
     }
+
+    @Test(expected = IOException.class)
+    public void testForEachIOException() throws Exception {
+        try (WriteCache cache = new WriteCache(allocator, 1024 * 1024, 16 * 1024)) {
+
+            for (int i = 0; i < 48; i++) {
+                boolean inserted = cache.put(1, i, Unpooled.wrappedBuffer(("test-" + i).getBytes()));
+                assertTrue(inserted);
+            }
+
+            assertEquals(48, cache.count());
+
+            cache.forEach(((ledgerId, entryId, entry) -> {
+                throw new IOException("test throw IOException.");
+            }));
+        }
+    }
 }
